@@ -28,6 +28,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "attribute.h"
 #include "gl_xlist.h"
 #include "gl_xmap.h"
 #include "gl_xset.h"
@@ -37,12 +38,12 @@
 
 #include "glcontainers.h"
 
-bool _GL_ATTRIBUTE_PURE string_equals (const void *s1, const void *s2)
+bool ATTRIBUTE_PURE string_equals (const void *s1, const void *s2)
 {
 	return STREQ ((const char *) s1, (const char *) s2);
 }
 
-size_t _GL_ATTRIBUTE_PURE string_hash (const void *s)
+size_t ATTRIBUTE_PURE string_hash (const void *s)
 {
 	return hash_pjw_bare (s, strlen ((const char *) s));
 }
