@@ -69,9 +69,9 @@ error_t argp_err_exit_status = FAIL;
 static const char args_doc[] = N_("FILE...");
 
 static struct argp_option options[] = {
-	{ "debug",	'd',	0,	0,	N_("emit debugging messages") },
-	{ "compatible",	'C',	0,	0,	N_("compatibility switch (ignored)"),	1 },
-	{ 0, 'h', 0, OPTION_HIDDEN, 0 }, /* compatibility for --help */
+	OPT ("debug", 'd', 0, N_("emit debugging messages")),
+	OPT ("compatible", 'C', 0, N_("compatibility switch (ignored)"), 1),
+	OPT_HELP_COMPAT,
 	{ 0 }
 };
 
